"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_utilities_1 = require("@contentstack/cli-utilities");
const utils_1 = require("../../../utils");
const base_command_1 = require("../../../base-command");
class RegionSetCommand extends base_command_1.BaseCommand {
    async run() {
        const { args, flags: regionSetFlags } = await this.parse(RegionSetCommand);
        let cda = regionSetFlags.cda;
        let cma = regionSetFlags.cma;
        let name = regionSetFlags.name;
        let uiHost = regionSetFlags['ui-host'];
        let developerHubUrl = regionSetFlags['developer-hub'];
        let personalizeUrl = regionSetFlags['personalize'];
        let launchHubUrl = regionSetFlags['launch'];
        let selectedRegion = args.region;
        if (!(cda && cma && uiHost && name) && !selectedRegion) {
            selectedRegion = await utils_1.interactive.askRegions();
        }
        if (selectedRegion === 'custom') {
            const selectedCustomRegion = await utils_1.interactive.askCustomRegion();
            name = selectedCustomRegion.name;
            cda = selectedCustomRegion.cda;
            cma = selectedCustomRegion.cma;
            uiHost = selectedCustomRegion.uiHost;
        }
        else if (selectedRegion === 'exit') {
            this.exit();
        }
        // Custom flag will get first priority over region argument
        if (cda && cma && uiHost && name) {
            try {
                if (!developerHubUrl) {
                    developerHubUrl = this.transformUrl(cma, 'developerhub-api');
                }
                if (!launchHubUrl) {
                    launchHubUrl = this.transformUrl(cma, 'launch-api');
                }
                if (!personalizeUrl) {
                    personalizeUrl = this.transformUrl(cma, 'personalize-api');
                }
                let customRegion = { cda, cma, uiHost, name, developerHubUrl, personalizeUrl, launchHubUrl };
                customRegion = utils_1.regionHandler.setCustomRegion(customRegion);
                await cli_utilities_1.authHandler.setConfigData('logout'); //Todo: Handle this logout flow well through logout command call
                cli_utilities_1.cliux.success(`Custom region has been set to ${customRegion.name}`);
                cli_utilities_1.cliux.success(`CMA HOST: ${customRegion.cma}`);
                cli_utilities_1.cliux.success(`CDA HOST: ${customRegion.cda}`);
                cli_utilities_1.cliux.success(`UI HOST: ${customRegion.uiHost}`);
                cli_utilities_1.cliux.success(`Developer Hub URL: ${customRegion.developerHubUrl}`);
                cli_utilities_1.cliux.success(`Personalize URL: ${customRegion.personalizeUrl}`);
                cli_utilities_1.cliux.success(`Launch URL: ${customRegion.launchHubUrl}`);
            }
            catch (error) {
                this.logger.error('failed to set the region', error);
                cli_utilities_1.cliux.error(`Failed to set region due to: ${error.message}`);
            }
        }
        else if (['NA', 'EU', 'AZURE-NA', 'AZURE-EU', 'GCP-NA'].includes(selectedRegion)) {
            const regionDetails = utils_1.regionHandler.setRegion(selectedRegion);
            await cli_utilities_1.authHandler.setConfigData('logout'); //Todo: Handle this logout flow well through logout command call
            cli_utilities_1.cliux.success(`Region has been set to ${regionDetails.name}`);
            cli_utilities_1.cliux.success(`CDA HOST: ${regionDetails.cda}`);
            cli_utilities_1.cliux.success(`CMA HOST: ${regionDetails.cma}`);
            cli_utilities_1.cliux.success(`UI HOST: ${regionDetails.uiHost}`);
            cli_utilities_1.cliux.success(`Developer Hub URL: ${regionDetails.developerHubUrl}`);
            cli_utilities_1.cliux.success(`Personalize URL: ${regionDetails.personalizeUrl}`);
            cli_utilities_1.cliux.success(`Launch URL: ${regionDetails.launchHubUrl}`);
        }
        else {
            cli_utilities_1.cliux.error(`Invalid region is given`);
        }
    }
    transformUrl(url, replacement) {
        let transformedUrl = url.replace('api', replacement);
        if (transformedUrl.startsWith('http')) {
            transformedUrl = transformedUrl.split('//')[1];
        }
        transformedUrl = transformedUrl.startsWith('dev11') ? transformedUrl.replace('dev11', 'dev') : transformedUrl;
        transformedUrl = transformedUrl.endsWith('io') ? transformedUrl.replace('io', 'com') : transformedUrl;
        return `https://${transformedUrl}`;
    }
}
exports.default = RegionSetCommand;
RegionSetCommand.description = 'Set region for CLI';
RegionSetCommand.flags = {
    cda: cli_utilities_1.flags.string({
        char: 'd',
        description: 'Custom host to set for content delivery API, if this flag is added then cma, ui-host and name flags are required',
        dependsOn: ['cma', 'ui-host', 'name'],
        parse: (0, cli_utilities_1.printFlagDeprecation)(['-d'], ['--cda']),
    }),
    cma: cli_utilities_1.flags.string({
        char: 'm',
        description: 'Custom host to set for content management API, , if this flag is added then cda, ui-host and name flags are required',
        dependsOn: ['cda', 'ui-host', 'name'],
        parse: (0, cli_utilities_1.printFlagDeprecation)(['-m'], ['--cma']),
    }),
    'ui-host': cli_utilities_1.flags.string({
        description: 'Custom UI host to set for CLI, if this flag is added then cda, cma and name flags are required',
        dependsOn: ['cda', 'cma', 'name'],
    }),
    name: cli_utilities_1.flags.string({
        char: 'n',
        description: 'Name for the region, if this flag is added then cda, cma and ui-host flags are required',
        dependsOn: ['cda', 'cma', 'ui-host'],
    }),
    'developer-hub': cli_utilities_1.flags.string({
        description: 'Custom host to set for Developer hub API',
    }),
    'personalize': cli_utilities_1.flags.string({
        description: 'Custom host to set for Personalize API',
    }),
    'launch': cli_utilities_1.flags.string({
        description: 'Custom host to set for Launch API',
    }),
};
RegionSetCommand.examples = [
    '$ csdx config:set:region',
    '$ csdx config:set:region NA',
    '$ csdx config:set:region EU',
    '$ csdx config:set:region AZURE-NA',
    '$ csdx config:set:region AZURE-EU',
    '$ csdx config:set:region GCP-NA',
    '$ csdx config:set:region --cma <contentstack_cma_endpoint> --cda <contentstack_cda_endpoint> --ui-host <contentstack_ui_host_endpoint> --name "India"',
    '$ csdx config:set:region --cma <contentstack_cma_endpoint> --cda <contentstack_cda_endpoint> --ui-host <contentstack_ui_host_endpoint> --name "India" --developer-hub <developer_hub_url>',
    '$ csdx config:set:region --cma <contentstack_cma_endpoint> --cda <contentstack_cda_endpoint> --ui-host <contentstack_ui_host_endpoint> --name "India" --personalize <personalize_url>',
    '$ csdx config:set:region --cma <contentstack_cma_endpoint> --cda <contentstack_cda_endpoint> --ui-host <contentstack_ui_host_endpoint> --name "India" --launch <launch_url>',
    '$ csdx config:set:region --cda <contentstack_cda_endpoint> --cma <contentstack_cma_endpoint> --ui-host <contentstack_ui_host_endpoint> --name "India" --developer-hub <developer_hub_url> --personalize <personalize_url> --launch <launch_url>',
];
RegionSetCommand.args = {
    region: cli_utilities_1.args.string({ description: 'Name for the region' }),
};
